/* gsl routines 
 * 
 * Copyright (C) 1996, 1997, 1998, 1999, 2000 Gerard Jungman, Brian Gough
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* THIS IS A STRIPPED VERSION OF GSL ROUTINES, INCLUDING ONLY
 * THE MINIMUM REQUIRED FUNCTIONALITY.
 */


#include <stdio.h>
#include <stdlib.h>
#include "gsl_routines.h"

#define BASE double
#define SHORT
#define ATOMIC double
#define MULTIPLICITY 1
#define IN_FORMAT "%lg"
#define OUT_FORMAT "%g"
#define ATOMIC_IO ATOMIC
#define ZERO 0
#define ONE 1

gsl_block *gsl_block_alloc (const size_t n);
void gsl_block_free (gsl_block * b);

gsl_block *
gsl_block_alloc (const size_t n)
{
  gsl_block * b;

  if (n == 0)
    {
      fprintf (stderr, "block length n must be positive integer\n");
    }

  b = (gsl_block *) malloc (sizeof (gsl_block));

  if (b == 0)
    {
      fprintf(stderr, "failed to allocate space for block struct\n");
    }

  b->data = (ATOMIC *) malloc (MULTIPLICITY * n * sizeof (ATOMIC));

  if (b->data == 0)
    {
      free (b);         /* exception in constructor, avoid memory leak */

      fprintf(stderr, "failed to allocate space for block data\n");
    }

  b->size = n;

  return b;
}

void
gsl_block_free (gsl_block * b)
{
  free (b->data);
  free (b);
}

gsl_vector *
gsl_vector_alloc (const size_t n)
{
  gsl_block * block;
  gsl_vector * v;

  if (n == 0)
      fprintf (stderr, "vector length n must be positive integer\n");

  v = (gsl_vector *) malloc (sizeof (gsl_vector));

  if (v == 0)
      fprintf (stderr, "failed to allocate space for vector struct\n");

  block = gsl_block_alloc (n);

  if (block == 0)
    {
      free (v) ;
      fprintf (stderr, "failed to allocate space for block\n");
    }
      
  v->data = block->data ;
  v->size = n;
  v->stride = 1;
  v->block = block;
  v->owner = 1;

  return v;
}

void
gsl_vector_free (gsl_vector * v)
{
  if (v->owner)
      gsl_block_free (v->block) ;
  free (v);
}

int
gsl_vector_swap_elements (gsl_vector * v, const size_t i, const size_t j)
{
  ATOMIC * data = v->data ;
  const size_t size = v->size ;
  const size_t stride = v->stride ;

  if (i >= size)
      fprintf(stderr, "first index is out of range\n");

  if (j >= size)
      fprintf(stderr, "second index is out of range\n");

  if (i != j)
    {
      const size_t s = MULTIPLICITY * stride ;
      size_t k ;

      for (k = 0; k < MULTIPLICITY; k++)
        {
          ATOMIC tmp = data[j*s + k];
          data[j*s+k] = data[i*s + k];
          data[i*s+k] = tmp;
        }
    }
  
  return GSL_SUCCESS;
}




gsl_matrix *
gsl_matrix_alloc (const size_t n1, const size_t n2)
{
  gsl_block * block;
  gsl_matrix * m;

  if (n1 == 0)
      fprintf(stderr, "matrix dimension n1 must be positive integer\n");
  else if (n2 == 0)
      fprintf(stderr, "matrix dimension n2 must be positive integer\n");

  m = (gsl_matrix *) malloc (sizeof (gsl_matrix));

  if (m == 0)
      fprintf(stderr, "failed to allocate space for matrix struct\n");

  /* FIXME: n1*n2 could overflow for large dimensions */

  block = gsl_block_alloc (n1 * n2) ;

  if (block == 0)
      fprintf(stderr, "failed to allocate space for block\n");

  m->data = block->data;
  m->size1 = n1;
  m->size2 = n2;
  m->tda = n2; 
  m->block = block;
  m->owner = 1;

  return m;
}



void
gsl_matrix_free (gsl_matrix * m)
{
  if (m->owner)
      gsl_block_free (m->block);

  free (m);
}



int
gsl_matrix_swap_columns (gsl_matrix * m,
                                     const size_t i, const size_t j)
{
  const size_t size1 = m->size1;
  const size_t size2 = m->size2;

  if (i >= size2)
      fprintf(stderr, "first column index is out of range\n");

  if (j >= size2)
      fprintf(stderr, "second column index is out of range\n");

  if (i != j)
    {
      ATOMIC *col1 = m->data + MULTIPLICITY * i;
      ATOMIC *col2 = m->data + MULTIPLICITY * j;
      
      size_t p;
      
      for (p = 0; p < size1; p++)
        {
          size_t k;
          size_t n = p * MULTIPLICITY * m->tda;
 
          for (k = 0; k < MULTIPLICITY; k++)
            {
              ATOMIC tmp = col1[n+k] ;
              col1[n+k] = col2[n+k] ;
              col2[n+k] = tmp ;
            }
        }
    }

  return GSL_SUCCESS;
}

